#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <crypt.h>
#include <pwd.h>
#include <grp.h>
#include <shadow.h>
#include <time.h>

#include <sys/types.h>
#include <sys/socket.h>

#include "snarecore.h"
#include "webserver.h"
#include "WebPages.h"

#define VERSION 2.3

extern char *getfqdn(char *);
extern void trim(char *);
extern int isheader(char *);
extern int iscomment(char *);
extern int getheader(char *);
extern char *gethostident(char *, char *, int);
extern char *getconfstring(char *string, char *file,int length);
extern int regmatch(const char *, const char *);
extern int regmatchi(const char *, const char *);
extern int getport(char *string);
extern int splitobjective(char *, char *, char *, char *, int *, int *);
extern void trimallwhitespace(char *);
extern int Send(int,char *,int);


// Make sure we return the size, or zero (for strings).
int HandleWebPages(char *HTTPBuffer, char *HTTPOutputBuffer, int size,
		   int http_listen_socket, int http_message_socket)
{
	int returncode = 0;
	char *ArgPosition;

	// Stuff without the header/footer
	if (!strcmp(HTTPBuffer, "/intersect.gif")) {
		return (InterSectImage(HTTPBuffer, HTTPOutputBuffer, size));
	} else {
		char *pBuffer = HTTPOutputBuffer;
		int length = 0;
		int psize = 0;

		ArgPosition = strstr(HTTPBuffer, "?");
		// No arguments passed?
		if (!ArgPosition) {
			// Set argument position to the beginning of the buffer.
			ArgPosition = HTTPBuffer;
		} else {
			ArgPosition++;
		}

		if (!strcmp(HTTPBuffer, "/Users")) {
			DisplayTextHeader(http_message_socket);
			ShowUsers(http_message_socket);
			return (-1);
		} else if (!strcmp(HTTPBuffer, "/Groups")) {
			DisplayTextHeader(http_message_socket);
			ShowGroups(http_message_socket);
			return (-1);
		} else if (!strcmp(HTTPBuffer, "/GroupMembers")) {
			DisplayTextHeader(http_message_socket);
			ShowGroupMembers(http_message_socket);
			return (-1);
		}

		returncode = DefaultHeader(HTTPBuffer, pBuffer, size);
		length = strlen(HTTPOutputBuffer);

		pBuffer += length;
		psize = size - length;
		if (psize < 0) {
			psize = 0;
		}

		// Web pages
		if (!strcmp(HTTPBuffer, "/network")) {
			returncode +=
			    Network_Config(ArgPosition, pBuffer, psize);
		} else if (!strcmp(HTTPBuffer, "/remote")) {
			returncode +=
			    Remote_Config(ArgPosition, pBuffer, psize);
		} else if (!strncmp(HTTPBuffer, "/setremote", 10)) {
			returncode +=
			    Remote_Set(ArgPosition, pBuffer, psize);
		} else if (!strncmp(HTTPBuffer, "/objective", 10)) {
			returncode +=
			    Objective_Config(ArgPosition, pBuffer, psize);
		} else if (!strncmp(HTTPBuffer, "/setobjective", 13)) {
			returncode +=
			    Objective_Display(HTTPBuffer, pBuffer, psize);
		} else if (!strncmp(HTTPBuffer, "/changeobjective", 16)) {
			returncode +=
			    Objective_Result(HTTPBuffer, pBuffer, psize);
		} else if (!strcmp(HTTPBuffer, "/restart")) {
			returncode +=
			    Restart(HTTPBuffer, pBuffer, psize,
				    http_listen_socket, http_message_socket);
		} else if (!strncmp(HTTPBuffer, "/setnetwork", 11)) {
			returncode += Network_Set(HTTPBuffer, pBuffer, psize);
		} else if (!strncmp(HTTPBuffer, "/status", 6)) {
			returncode += Status_Page(ArgPosition, pBuffer, psize);
		} else {
			returncode = +Status_Page(ArgPosition, pBuffer, psize);
		}

		pBuffer = HTTPOutputBuffer;
		length = strlen(HTTPOutputBuffer);
		pBuffer += length;
		psize = size - length;
		if (psize < 0) {
			psize = 0;
		}

		returncode += DefaultFooter(HTTPBuffer, pBuffer, psize);
	}

	return (returncode);
}

int Status_Page(char *source, char *dest, int size)
{
	strncpy(dest,
		"<HTML><BODY><H1><CENTER>Welcome to SNARE for Solaris version 2.3</H1><P>Please select from the menu on the left.</CENTER><p>",
		size);

	// Ok.. we're cheating here. The only way we can get here is if we're running.
	strncat(dest,
		"Status: <font color=green>Snare for Solaris currently running.</font>",
		size - strlen(dest));

	strncat(dest, "</BODY></HTML>", size - strlen(dest));
	return (0);
}

int ShowUsers(int http_socket)
{
	struct passwd *user;
	struct spwd *sp;
	int retval;
	char temp[1024];
	char hostid[MAX_HOSTID];

	getfqdn(hostid);

	snprintf(temp, sizeof (temp), "HOST: %s\n", hostid);
	retval = Send(http_socket, temp, strlen(temp));
	setpwent();
	while ((user = (struct passwd *) getpwent()) != (struct passwd *) NULL) {
		snprintf(temp, sizeof (temp), "%s\t%d\t%s", user->pw_name,
			 (int)user->pw_uid, user->pw_gecos);
		retval = Send(http_socket, temp, strlen(temp));

		if ((sp = getspnam(user->pw_name)) != (struct spwd *) NULL) {
			if (sp->sp_max > 0
			    && ((time(NULL) / 86400) >
				(sp->sp_lstchg + sp->sp_max))) {

				strncpy(temp, "\tPASSWD_EXPIRED",
					sizeof (temp));
				retval =
				    Send(http_socket, temp, strlen(temp));
			}
			if (sp->sp_lstchg > 0 && sp->sp_max > 0
			    && sp->sp_lstchg > 0
			    && ((time(NULL) / 86400) >
				(sp->sp_lstchg + sp->sp_max + sp->sp_inact))) {

				strncpy(temp, "\tACCOUNT_INACTIVE",
					sizeof (temp));
				retval =
				    Send(http_socket, temp, strlen(temp));
			}

			if (sp->sp_expire > 0
			    && (time(NULL) / 86400) > sp->sp_expire) {
				strncpy(temp, "\tACCOUNT_EXPIRED",
					sizeof (temp));
				retval =
				    Send(http_socket, temp, strlen(temp));
			}
			if (!strcmp(sp->sp_pwdp, "NP")) {
				strncpy(temp, "\tACCOUNT_DISABLED",
					sizeof (temp));

				retval =
				    Send(http_socket, temp, strlen(temp));
			} else if (!strcmp(sp->sp_pwdp, "*LK*")) {
				strncpy(temp, "\tACCOUNT_LOCKED",
					sizeof (temp));

				retval =
				    Send(http_socket, temp, strlen(temp));
			} else if (!strcmp(sp->sp_pwdp, "")) {
				strncpy(temp, "\tNO_PASSWD", sizeof (temp));

				retval =
				    Send(http_socket, temp, strlen(temp));
			}

		}
		retval = Send(http_socket, "\n", strlen("\n"));
	}
	endpwent();

	return (1);
}

int ShowGroups(int http_socket)
{
	struct group *groupp;
	int retval;
	char temp[1024];
	char hostid[MAX_HOSTID];

	getfqdn(hostid);

	snprintf(temp, sizeof(temp), "HOST: %s\n", hostid);
	retval = Send(http_socket, temp, strlen(temp));
	setgrent();
	while ((groupp =
		(struct group *) getgrent()) != (struct group *) NULL) {
		snprintf(temp, 1024, "%s\t%d\n", groupp->gr_name,
			 (int)groupp->gr_gid);
		retval = Send(http_socket, temp, strlen(temp));
	}
	endgrent();

	return (1);
}

int ShowGroupMembers(int http_socket)
{
	struct group *groupp;
	int retval;
	char temp[1024];
	int firstmember = 1;
	char hostid[MAX_HOSTID];

	getfqdn(hostid);

	snprintf(temp, sizeof (temp), "HOST: %s\n", hostid);
	retval = Send(http_socket, temp, strlen(temp));
	setgrent();
	while ((groupp = (struct group *) getgrent()) != (struct group *) NULL) {
		firstmember = 1;
		snprintf(temp, sizeof (temp), "%s\t%d\t", groupp->gr_name,
			 (int)groupp->gr_gid);
		retval = Send(http_socket, temp, strlen(temp));
		while (*(groupp->gr_mem) != NULL) {
			if (firstmember) {
				snprintf(temp, sizeof (temp), "%s",
					 *((groupp->gr_mem)++));
				firstmember = 0;
			} else {
				snprintf(temp, sizeof (temp), ",%s",
					 *((groupp->gr_mem)++));
			}
			retval = Send(http_socket, temp, strlen(temp));
		}
		retval = Send(http_socket, "\n", strlen("\n"));
	}
	endgrent();

	return (1);
}

int Network_Config(char *source, char *dest, int size)
{
	struct Reg_Config config_struct;
	int dw_config_error;
	char str_DestPort[10];
	char str_conferr[10], str_neterr[10];

	strcpy(config_struct.str_ClientName, "");
	strcpy(config_struct.str_FileDestination, "");
	strcpy(config_struct.str_NetworkDestination, "");
	config_struct.dw_stdout = 0;

	dw_config_error = Read_Config_From_File(&config_struct);

	// This function will display the form used to set the audit configuration
	// The result of the form will be sent to "network_set"
	strncpy(dest,
		"<form action=setnetwork><h1><center>SNARE Configuration</h1>",
		size);

	// Will display an error if unable to completely read from the registry
	if (dw_config_error > 0) {
		dw_config_error += WEB_READ_CONFIG_ERROR_CODE;
		snprintf(str_conferr, 10, "%d", dw_config_error);

		strncat(dest,
			"<br><b>NOTE: Some errors were encountered in reading the file. Default values "
			"may be used.<br> Report error: ", size - strlen(dest));
		strncat(dest, str_neterr, size - strlen(dest));
		strncat(dest, ".", size - strlen(dest));
		strncat(dest, str_conferr, size - strlen(dest));
		strncat(dest, "</b><br>", size - strlen(dest));
	}

	strncat(dest,
		"<br>The following network configuration parameters of the SNARE unit are set to the following values:<br><br>"
		"<table  width=70% border=0>"
		"<tr bgcolor=#FFFFCC><td>Clientname</td><td><input type=text name=str_ClientName size=25 value=\"",
		size - strlen(dest));

	strncat(dest, config_struct.str_ClientName, size - strlen(dest));
	strncat(dest, "\"></td></tr>", size - strlen(dest));

	strncat(dest,
		"<tr bgcolor=#FFFFBB><td>Destination Server address </td><td><input type=text name=str_NetworkDestination size=25 value=\"",
		size - strlen(dest));
	strncat(dest, config_struct.str_NetworkDestination,
		size - strlen(dest));
	strncat(dest, "\"></td></tr>", size - strlen(dest));

	strncat(dest,
		"<tr bgcolor=#FFFFCC><td>Destination UDP Port</td><td><input type=text name=dw_DestPort size=8 value=\"",
		size - strlen(dest));

	if (config_struct.dw_DestPort) {
		snprintf(str_DestPort, 10, "%d", config_struct.dw_DestPort);
	} else {
		str_DestPort[0] = '\0';
	}

	strncat(dest, str_DestPort, size - strlen(dest));
	strncat(dest, "\"></td></tr>", size - strlen(dest));

	strncat(dest,
		"<tr bgcolor=#FFFFBB><td>Destination FileName </td><td><input type=text name=str_FileDestination size=25 value=\"",
		size - strlen(dest));
	strncat(dest, config_struct.str_FileDestination, size - strlen(dest));
	strncat(dest, "\"></td></tr>", size - strlen(dest));

	// Need to convert this next section to YES or NO
	strncat(dest,
		"<tr bgcolor=#FFFFBB><td>Audit to STDOUT </td><td><input type=checkbox name=dw_StdoutDestination",
		size - strlen(dest));
	if (config_struct.dw_stdout != 0)
		strncat(dest, " checked", size - strlen(dest));
	strncat(dest, "></td></tr>", size - strlen(dest));

	strncat(dest, "</table><br>", size - strlen(dest));
	strncat(dest, "<input type=submit value=\"Change Configuration\">    ",
		size - strlen(dest));
	strncat(dest, "<input type=reset value=\"Reset Form\"></form>",
		size - strlen(dest));

	return (0);
}

int Network_Set(char *source, char *dest, int size)
{
	char *psource = source;
	char Variable[100], web_port[100];
	char Argument[100];
	struct Reg_Config config_struct;
	int dw_error_config = 0;

	// This function will display that the remote audit configurations have been changed, or there have been errors
	strncpy(dest, "<h1><center>SNARE Configuration</h1>", size);

	// Note that all the possible variables do NOT have to be in the URL. The ones that are selected
	// via a checkbox will not be listed if the checkbox has been deselected.
	// Checking is limited to ensuring the Detsination port in the range 1-65535.
	// The variable associated with the checkbox (dw_Syslog) must be
	// exactly "on" or it will be defaulted to "off".
	// str_Destination,str_Delimiter_Text,str_ClientName can be anything it wants to be, so long 
	// as it is within size bounds.

	config_struct.dw_DestPort = -1;
	strncpy(config_struct.str_NetworkDestination, "",
		sizeof (config_struct.str_NetworkDestination));
	strncpy(config_struct.str_ClientName, "",
		sizeof (config_struct.str_ClientName));
	strncpy(web_port, "", sizeof (web_port));
	strncpy(config_struct.str_FileDestination, "",
		sizeof (config_struct.str_FileDestination));
	config_struct.dw_stdout = 0;

	while ((psource =
		GetNextArgument(psource, Variable, sizeof (Variable), Argument,
				sizeof (Argument))) != (char *) NULL) {

		if (strstr(Variable, "str_ClientName") != NULL) {
			strncpy(config_struct.str_ClientName, Argument,
				sizeof (config_struct.str_ClientName));
		}

		if (strstr(Variable, "str_NetworkDestination") != NULL) {
			strncpy(config_struct.str_NetworkDestination, Argument,
				sizeof (config_struct.str_NetworkDestination));
		}

		if (strstr(Variable, "dw_DestPort") != NULL) {
			strncpy(web_port, Argument, sizeof (web_port));
		}

		if (strstr(Variable, "str_FileDestination") != NULL) {
			strncpy(config_struct.str_FileDestination, Argument,
				sizeof (config_struct.str_FileDestination));
		}
		if (strstr(Variable, "dw_StdoutDestination") != NULL) {
			if (strcmp(Argument, "on") == 0) {
				config_struct.dw_stdout = 1;
			} else {
				config_struct.dw_stdout = 0;
			}
		}
	}

	config_struct.dw_DestPort = atoi(web_port);
	if (strlen(config_struct.str_NetworkDestination)
	    && ((config_struct.dw_DestPort < 1)
	    || (config_struct.dw_DestPort > 65535))) {
		strncat(dest,
			"The Destination Port value must be between 1 and 65535. Use the 'back' button to change the value.",
			size - strlen(dest));
	} else {
		void *rampointer = (void *) NULL;
		char *position;
		char inputbuffer[MAX_AUDIT_CONFIG_LINE];
		int headertype = 0;
		FILE *configfile;

		rampointer = Load_Config_File();

		if (!rampointer) {
			dw_error_config = 1;
		} else {
			int size = 0;
			int wroteconfig = 0;
			int skip = 0;

			position = (char *) rampointer;

			configfile = fopen(CONFIG_FILENAME, "w");
			if (!configfile) {
				dw_error_config = 1;
				strncat(dest,
					"<br><b>NOTE: Could not open the configuration file for writing. Please verify the permissions set on the audit config file.",
					size - strlen(dest));
				Clear_Config_File(rampointer);
				return (0);
			}

			while ((size =
				Grab_RAMConfig_Line(position, inputbuffer,
						    MAX_AUDIT_CONFIG_LINE))) {
				trim(inputbuffer);

				// Is this line a header?
				if (isheader(inputbuffer) || skip) {
					// Jump over any lines in the headers that we're writing out.
					if (!isheader(inputbuffer)) {
						position += size;
						continue;
					}
					fprintf(configfile, "%s\n",
						inputbuffer);

					headertype = getheader(inputbuffer);
					if (headertype == CONFIG_HOSTID) {
						wroteconfig += 1;
						// WRITE OUT NEW hostid here.
						if (strlen
						    (config_struct.
						     str_ClientName)) {
							fprintf(configfile,
								"	name=%s\n",
								config_struct.
								str_ClientName);
						}
						skip = 1;
					} else if (headertype == CONFIG_OUTPUT) {
						wroteconfig += 2;
						if (strlen
						    (config_struct.
						     str_FileDestination)) {
							fprintf(configfile,
								"	file=%s\n",
								config_struct.
								str_FileDestination);
						}
						if (strlen
						    (config_struct.
						     str_NetworkDestination)
						    && config_struct.
						    dw_DestPort) {
							fprintf(configfile,
								"	network=%s:%d\n",
								config_struct.
								str_NetworkDestination,
								config_struct.
								dw_DestPort);

						}
						if (config_struct.dw_stdout) {
							fprintf(configfile,
								"	file=stdout\n");
						}
						skip = 1;
					} else {
						skip = 0;
					}
				} else {

					// Print this line to file.
					if (iscomment(inputbuffer)
					    || !strlen(inputbuffer)) {
						fprintf(configfile, "%s\n",
							inputbuffer);
					} else {
						fprintf(configfile, "	%s\n",
							inputbuffer);
					}
				}
				position += size;
			}

			if (!wroteconfig || wroteconfig == 2) {
				if (strlen(config_struct.str_ClientName)) {
					fprintf(configfile,
						"\n[HostID]\n	name=%s\n",
						config_struct.str_ClientName);
				}
			}
			if (!wroteconfig || wroteconfig == 1) {
				fprintf(configfile, "\n[Output]\n");
				if (strlen(config_struct.str_FileDestination)) {
					fprintf(configfile, "	file=%s\n",
						config_struct.
						str_FileDestination);
				}
				if (strlen(config_struct.str_NetworkDestination)
				    && config_struct.dw_DestPort) {
					fprintf(configfile,
						"	network=%s:%d\n",
						config_struct.
						str_NetworkDestination,
						config_struct.dw_DestPort);
				}
				if (config_struct.dw_stdout) {
					fprintf(configfile, "	file=stdout\n");
				}
				fprintf(configfile, "\n");
			}

			if (fclose(configfile)) {
				dw_error_config = 1;
				strncat(dest,
					"<br><b>NOTE: Could not write to the configuration file. Does the system have enough free disk space?",
					size - strlen(dest));
				Clear_Config_File(rampointer);
				return (0);
			}

			Clear_Config_File(rampointer);
		}

		if (dw_error_config != 0) {
			strncat(dest, "Values have NOT been changed.",
				size - strlen(dest));
		} else {
			strncat(dest, "Values have been changed.",
				size - strlen(dest));
		}
	}

	return (0);

}

int Remote_Config(char *source, char *dest, int size)
{
	struct Reg_Remote remote_struct;
	int dw_remote_error;
	char str_WebPort[10];
	char str_remerr[10];

	dw_remote_error = Read_Remote_From_File(&remote_struct);

	// This function will display the form used to set the remote audit configuration
	strncpy(dest,
		"<form action=setremote><h1><center>SNARE Remote Control Configuration</h1>",
		size);

	// Will display an error if unable to completely read from the registry
	if (dw_remote_error > 0) {
		dw_remote_error += WEB_READ_REMOTE_ERROR_CODE;
		// itoa(dw_remote_error,str_remerr,10);
		snprintf(str_remerr, 10, "%d", dw_remote_error);

		strncat(dest,
			"<br><b>NOTE: Some errors were encountered in reading the registry. Default values "
			"may be used.<br> Report error: ", size - strlen(dest));
		strncat(dest, str_remerr, size - strlen(dest));
		strncat(dest, "</b><br>", size - strlen(dest));
	}
	strncat(dest,
		"<br>The following remote control configuration parameters of the SNARE unit is set to the following values:<br><br>"
		"<table  width=70% border=0>"
		"<tr bgcolor=#FFFFCC><td>Allow remote control of SNARE agent</td><td><input type=checkbox name=dw_Allow",
		size - strlen(dest));

	// Need to convert this next section to YES or NO
	if (remote_struct.dw_Allow != 0) {
		strncat(dest, " checked", size - strlen(dest));
	}

	strncat(dest, "></td></tr><tr bgcolor=#FFFFFF><td><br></td></tr>",
		size - strlen(dest));
	strncat(dest,
		"<tr bgcolor=#FFFFBB><td>Restrict remote control of SNARE agent to certain hosts </td><td><input type=checkbox name=dw_Restrict",
		size - strlen(dest));

	// Need to convert this next section to YES or NO
	if (remote_struct.dw_Restrict != 0)
		strncat(dest, " checked", size - strlen(dest));
	strncat(dest, "></td></tr>", size - strlen(dest));

	strncat(dest,
		"<tr bgcolor=#FFFFCC><td>IP Address allowed to remote control SNARE </td><td><input type=text name=str_RestrictIP size=12 value=\"",
		size - strlen(dest));
	strncat(dest, remote_struct.str_RestrictIP, size - strlen(dest));
	strncat(dest, "\"></td></tr>", size - strlen(dest));

	strncat(dest,
		"<tr bgcolor=#FFFFBB><td>Require a password for remote control? </td><td><input type=checkbox name=dw_Password",
		size - strlen(dest));

	// Need to convert this next section to YES or NO
	if (remote_struct.dw_Password != 0)
		strncat(dest, " checked", size - strlen(dest));
	strncat(dest, "></td></tr>", size - strlen(dest));

	strncat(dest,
		"<tr bgcolor=#FFFFCC><td>Password to allow remote control of SNARE </td><td><input type=password name=str_Password size=12 value=\"",
		size - strlen(dest));
	strncat(dest, remote_struct.str_Password, size - strlen(dest));
	strncat(dest, "\"><input type=hidden name=str_OldPassword value=\"",size -strlen(dest));
	strncat(dest, remote_struct.str_Password, size - strlen(dest));
	strncat(dest, "\"></td></tr>", size - strlen(dest));

	strncat(dest,
		"<tr bgcolor=#FFFFCC><td>Web Server Port </td><td><input type=text name=dw_WebPort size=8 value=\"",
		size - strlen(dest));
	snprintf(str_WebPort, 10, "%d", remote_struct.dw_WebPort);
	strncat(dest, str_WebPort, size - strlen(dest));
	strncat(dest, "\"></td></tr>", size - strlen(dest));

	strncat(dest, "</table><br>", size - strlen(dest));
	strncat(dest, "<input type=submit value=\"Change Configuration\">    ",
		size - strlen(dest));
	strncat(dest, "<input type=reset value=\"Reset Form\"></form>",
		size - strlen(dest));

	return (0);
}

int Remote_Set(char *source, char *dest, int size)
{
	char *psource = source;
	char Variable[100], web_port[100];
	char Argument[100];
	char OldPassword[100]="";
	struct Reg_Remote remote_struct;
	int dw_error = 0;

	//This function will display that the remote audit configurations have been changed, or there have been errors
	strncpy(dest, "<h1><center>SNARE Remote Control Configuration</h1>",
		size);

	// Note that all the possible variables do NOT have to be in the URL. The ones that are selected
	// via a checkbox will not be listed if the checkbox has been deselected.
	// Also be aware that there may not be any arguments for this objective. If a sysadmin does not want
	// remote control, then there will be no arguments.
	// Hence: Checking is limited to Webport in the range 1-65535 only if portchange is "on"
	// The three variable associated with the checkboxes (dw_Allow, dw_Restrict, and dw_PortChange) must be
	// exactly "on" or they will be defaulted to "off".
	// str_RestrictIP can be anything it wants to be, so long as it is within size bounds.

	remote_struct.dw_WebPort = -1;
	remote_struct.dw_Allow = 0;
	remote_struct.dw_Password = 0;
	remote_struct.dw_Restrict = 0;

	while ((psource =
		GetNextArgument(psource, Variable, sizeof (Variable), Argument,
				sizeof (Argument))) != (char *) NULL) {

		if (strstr(Variable, "dw_WebPort") != NULL) {
			strncpy(web_port, Argument, sizeof (web_port));
		}
		if (strstr(Variable, "str_RestrictIP") != NULL) {
			strncpy(remote_struct.str_RestrictIP, Argument,
				sizeof (remote_struct.str_RestrictIP));
		}
		if (strstr(Variable, "str_Password") != NULL) {
			strncpy(remote_struct.str_Password, Argument,
				sizeof (remote_struct.str_Password));
		}

		if (strstr(Variable, "str_OldPassword") != NULL) {
			strncpy(OldPassword, Argument,
				sizeof (OldPassword));
		}
		if (strstr(Variable, "dw_Allow") != NULL) {
			if (strcmp(Argument, "on") == 0)
				remote_struct.dw_Allow = 1;
			else
				remote_struct.dw_Allow = 0;
		}
		if (strstr(Variable, "dw_Password") != NULL) {
			if (strcmp(Argument, "on") == 0) {
				remote_struct.dw_Password = 1;
			} else {
				remote_struct.dw_Password = 0;
			}
		}
		if (strstr(Variable, "dw_Restrict") != NULL) {
			if (strcmp(Argument, "on") == 0)
				remote_struct.dw_Restrict = 1;
			else
				remote_struct.dw_Restrict = 0;
		}
	}

	remote_struct.dw_WebPort = atoi(web_port);
	if ((remote_struct.dw_WebPort < 1)
	    || (remote_struct.dw_WebPort > 65535)) {
		strncat(dest,
			"The Web Port value must be between 1 and 65535. Use the 'back' button to change the value.",
			size - strlen(dest));
	} else {
		void * rampointer = (void *) NULL;
		char * position;
		char inputbuffer[MAX_AUDIT_CONFIG_LINE];
		int headertype = 0;
		FILE * configfile=(FILE *)NULL;

		rampointer = Load_Config_File();

		if (!rampointer) {
			dw_error = 1;
		} else {
			int size = 0;
			int wroteconfig = 0;
			int skip = 0;

			position = (char *) rampointer;

			configfile = fopen(CONFIG_FILENAME, "w");
			if (!configfile) {
				dw_error = 1;
				strncat(dest,
					"<br><b>NOTE: Could not open the configuration file for writing. Please verify the permissions set on the audit config file.",
					size - strlen(dest));
				Clear_Config_File(rampointer);
				return (0);
			}

			while ((size =
				Grab_RAMConfig_Line(position, inputbuffer,
						    MAX_AUDIT_CONFIG_LINE))) {

				trim(inputbuffer);

				// Is this line a header?
				if (isheader(inputbuffer) || skip) {
					// Jump over any lines in the headers that we're writing out.
					if (!isheader(inputbuffer)) {
						position += size;
						continue;
					}

					fprintf(configfile, "%s\n",
						inputbuffer);

					headertype = getheader(inputbuffer);

					if (headertype == CONFIG_REMOTE) {
						wroteconfig = 1;

						if (remote_struct.dw_Allow) {
							fprintf(configfile,
								"	allow=1\n");
						}
						if (remote_struct.dw_WebPort) {
							fprintf(configfile,
								"	listen_port=%d\n",
								remote_struct.
								dw_WebPort);
						}
						if (strlen
						    (remote_struct.
						     str_RestrictIP)
						    && remote_struct.
						    dw_Restrict) {
							fprintf(configfile,
								"	restrict_ip=%s\n",
								remote_struct.
								str_RestrictIP);
						}
						if (strlen
						    (remote_struct.str_Password)
						    && remote_struct.
						    dw_Password) {
							if(strcmp(remote_struct.str_Password,OldPassword)) {
								// The password has changed. Encrypt it.
								fprintf(configfile,
									"	accesskey=%s\n",
									(char *)crypt(remote_struct.str_Password,SALT));
							} else {
								// Otherwise, leave it in peace
								fprintf(configfile,
									"	accesskey=%s\n",
									remote_struct.str_Password);
							}
						}
						skip = 1;
					} else {
						skip = 0;
					}
				} else {
					// Print this line to file.
					if (iscomment(inputbuffer)
					    || !strlen(inputbuffer)) {
						fprintf(configfile, "%s\n",
							inputbuffer);
					} else {
						fprintf(configfile, "	%s\n",
							inputbuffer);
					}
				}
				position += size;
			}

			if (!wroteconfig) {
				fprintf(configfile, "\n[Remote]\n");

				if (remote_struct.dw_Allow) {
					fprintf(configfile, "	allow=1\n");
				}
				if (remote_struct.dw_WebPort) {
					fprintf(configfile,
						"	listen_port=%d\n",
						remote_struct.dw_WebPort);
				}
				if (strlen(remote_struct.str_RestrictIP)
				    && remote_struct.dw_Restrict) {
					fprintf(configfile,
						"	restrict_ip=%s\n",
						remote_struct.str_RestrictIP);
				}
				if (strlen(remote_struct.str_Password)
				    && remote_struct.dw_Password) {
					fprintf(configfile, "	accesskey=%s\n",
						(char *)crypt(remote_struct.str_Password,SALT));
				}
				fprintf(configfile, "\n");
			}

			if (fclose(configfile)) {
				dw_error = 1;
				strncat(dest,
					"<br><b>NOTE: Could not write to the configuration file. Does the system have enough free disk space?.",
					size - strlen(dest));
				Clear_Config_File(rampointer);
				return (0);
			}

			Clear_Config_File(rampointer);
		}

		if (dw_error != 0) {
			strncat(dest,
				"Remote Control Values have NOT been changed. Report error: ",
				size - strlen(dest));
		} else {
			strncat(dest,
				"Remote Control Values have been changed.",
				size - strlen(dest));
		}
	}

	return (0);
}

int Objective_Config(char *source, char *dest, int size)
{
	struct Reg_Objective reg_objective;
	int i_objective_count = 0;
	char str_obj_count[10];
	char str_user_match_metachar_remove[SIZE_OF_USERMATCH * 2];
	char str_eventid_match_metachar_remove[SIZE_OF_EVENTIDMATCH * 2];
	char str_general_match_metachar_remove[SIZE_OF_GENERALMATCH * 2];

	FILE *configfile = (FILE *) NULL;

	strncpy(dest,
		"<form action=setobjective><H1><CENTER>SNARE Filtering Objectives Configuration</H1>",
		size);

	configfile = Find_First_Objective();

	if (configfile) {
		strncat(dest,
			"<br>The following filtering objectives of the SNARE unit are active:<br><br>"
			"<table  width=100% border=1>", size - strlen(dest));

		strncat(dest,
			"<tr bgcolor=#FFFFBB><center><td><b>Action Required</b></td><td><b>Criticality</b></td>"
			"<td><b>Event ID Match</b></td><td><b>User Match</b></td><td><b>Search Term (regular expression)</b>"
			"</td></center></tr>", size - strlen(dest));

		while (Get_Next_Objective(configfile, &reg_objective)) {
			snprintf(str_obj_count, 10, "%d", i_objective_count);

			if ((i_objective_count) == 0)
				strncat(dest,
					"<tr bgcolor=#FFFFCC><td><input type=submit name=",
					size - strlen(dest));
			else
				strncat(dest,
					"<tr bgcolor=#FFFFBB><td><input type=submit name=",
					size - strlen(dest));

			strncat(dest, str_obj_count, size - strlen(dest));
			strncat(dest, " value=Delete>     ",
				size - strlen(dest));

			strncat(dest, "<input type=submit name=",
				size - strlen(dest));
			strncat(dest, str_obj_count, size - strlen(dest));
			strncat(dest, " value=Modify>", size - strlen(dest));
			strncat(dest, "</td><td>", size - strlen(dest));

			strncat(dest, reg_objective.str_critic,
				size - strlen(dest));
			strncat(dest, "</td><td>", size - strlen(dest));

			// Debracket the strings in here. For HTML display purposes, the HTML metacharacters
			// need to be replaced. This is done with the "debracket" routine.
			// Note that the new strings are allowed to be twice as long as the real strings
			debracket(reg_objective.str_user_match,
				  str_user_match_metachar_remove,
				  SIZE_OF_USERMATCH * 2);
			debracket(reg_objective.str_eventid_match,
				  str_eventid_match_metachar_remove,
				  SIZE_OF_EVENTIDMATCH * 2);
			debracket(reg_objective.str_general_match,
				  str_general_match_metachar_remove,
				  SIZE_OF_GENERALMATCH * 2);

			if (strlen(reg_objective.str_eventid_match) == 0) {
				strncat(dest, "&nbsp", size - strlen(dest));
			} else {
				strncat(dest, str_eventid_match_metachar_remove,
					size - strlen(dest));
			}
			strncat(dest, "</td><td>      ", size - strlen(dest));

			if (strlen(reg_objective.str_user_match) == 0) {
				strncat(dest, "&nbsp", size - strlen(dest));
			} else {
				strncat(dest, str_user_match_metachar_remove,
					size - strlen(dest));
			}
			strncat(dest, "</td><td>", size - strlen(dest));

			if (strlen(reg_objective.str_general_match) == 0) {
				strncat(dest, "&nbsp", size - strlen(dest));
			} else {
				strncat(dest, str_general_match_metachar_remove,
					size - strlen(dest));
			}
			strncat(dest, "</td></tr>", size - strlen(dest));

			i_objective_count++;
		}
		Close_Objective_File(configfile);
		strncat(dest, "</table><br>", size - strlen(dest));
	} else {
		strncat(dest,
			"<br>There are no current filtering objectives active.<br><br>",
			size - strlen(dest));
	}

	strncat(dest, "Select this button to add a new objective.  ",
		size - strlen(dest));
	strncat(dest, "<input type=submit name=0", size - strlen(dest));
	strncat(dest, " value=Add>", size - strlen(dest));

	return (0);
}

int Objective_Display(char *source, char *dest, int size)
{
	struct Reg_Objective reg_objective;
	int dw_objective_error = 0, dw_objective_delete_error = 0;
	char str_objerr[10];
	int i_set = 0, i_objective_count = 0, i_type = 0;
	char *psource = source, Variable[100], Argument[100];
	char str_temp[20], str_temp_objective[10];

	// This function will display an existing, or a blank, objective
	strncpy(dest,
		"<form action=changeobjective><h1><center>SNARE Filtering Objective Configuration</h1>",
		size);

	// Determine whether the objective will be modified or deleted
	while ((psource =
		GetNextArgument(psource, Variable, sizeof (Variable), Argument,
				sizeof (Argument))) != (char *) NULL) {
		if (strstr(Argument, "Delete") != NULL) {
			sscanf(Variable, "%20[^?]?%10[^\n]\n", str_temp,
			       str_temp_objective);
			i_type = 0;
			break;
		}
		if (strstr(Argument, "Modify") != NULL) {
			sscanf(Variable, "%20[^?]?%10[^\n]\n", str_temp,
			       str_temp_objective);
			i_type = 1;
			break;
		}
		if (strstr(Argument, "Add") != NULL) {
			strncpy(str_temp_objective, "-2",
				sizeof (str_temp_objective));
			i_type = 2;
			break;
		}
	}

	// Extract the objective number. I have to do this stuff, because atoi returns 0 if it cannot convert the string
	if (strcmp(str_temp_objective, "0") == 0)
		i_objective_count = -1;
	else
		i_objective_count = atoi(str_temp_objective);

	// If the objective number could not be successfully extracted, return immediately.
	if (i_objective_count == 0) {
		strncat(dest,
			"<br><b>NOTE: It appears the URL is encoded incorrectly.",
			size - strlen(dest));
		return 0;
	}

	if (i_objective_count == -1)
		i_objective_count = 0;

	// If the objective is being modified or added
	if (i_type > 0) {
		if (i_type == 1) {
			int count = 0;
			int returncode;
			FILE *configfile;
			configfile = Find_First_Objective();
			while ((returncode =
				Get_Next_Objective(configfile, &reg_objective)))
			{
				if (count == i_objective_count)
					break;
				count++;
			}
			if (!(count == i_objective_count && returncode)) {
				dw_objective_error =
				    WEB_READ_OBJECTIVE_ERROR_CODE;
			}
			Close_Objective_File(configfile);
		} else {
			// Defaults
			strncpy(reg_objective.str_event_type, SUCCESS_TOKEN,
				sizeof (reg_objective.str_event_type));
			strncpy(reg_objective.str_critic, CRITICAL_TOKEN,
				sizeof (reg_objective.str_critic));
			strncpy(reg_objective.str_eventid_match, LOGONOFF_TOKEN,
				sizeof (reg_objective.str_eventid_match));
			strncpy(reg_objective.str_general_match,
				"\"Enter the non-header search term here\"",
				sizeof (reg_objective.str_general_match));
			strncpy(reg_objective.str_user_match,
				"\"Enter the user match here\"",
				sizeof (reg_objective.str_user_match));
		}

		// Will display an error if unable to completely read from the config file
		if (dw_objective_error > 0) {
			dw_objective_error += WEB_READ_OBJECTIVE_ERROR_CODE;
			snprintf(str_objerr, 10, "%d", dw_objective_error);

			strncat(dest,
				"<br><b>NOTE: Some errors were encountered in reading the configuration file. Default values "
				"may be used.<br> Report error: ",
				size - strlen(dest));
			strncat(dest, str_objerr, size - strlen(dest));
			strncat(dest, "</b><br>", size - strlen(dest));
		}

		strncat(dest,
			"<br>The following parameters of the SNARE objective may be set:<br><br>"
			"<table  width=100% border=0>", size - strlen(dest));

		// Identify the high level event. Note that there is a table within a table in these radio buttons.
		i_set = 0;

		strncat(dest,
			"<tr bgcolor=#FFFFBB><td>Identify the high level event</td><td><table  width=100% border=0><tr>",
			size - strlen(dest));
		strncat(dest,
			"<td><input type=radio name=str_eventid_match value=",
			size - strlen(dest));
		strncat(dest, LOGONOFF_TOKEN, size - strlen(dest));

		if (strcmp(reg_objective.str_eventid_match, LOGON_LOGOFF_EVENTS)
		    == 0) {
			strncat(dest, " checked", size - strlen(dest));
			i_set = 1;
		}
		strncat(dest,
			">User Logon or Logoff  </td><td><input type=radio name=str_eventid_match value=",
			size - strlen(dest));
		strncat(dest, FILE_READ_TOKEN, size - strlen(dest));
		if (strcmp
		    (reg_objective.str_eventid_match,
		     FILE_READ_EVENTS) == 0) {
			strncat(dest, " checked", size - strlen(dest));
			i_set = 1;
		}
		strncat(dest,
			">Open a file/dir for reading only  </td><td><input type=radio name=str_eventid_match value=",
			size - strlen(dest));
		strncat(dest, FILE_WRITE_TOKEN, size - strlen(dest));
		if (strcmp
		    (reg_objective.str_eventid_match,
		     FILE_WRITE_EVENTS) == 0) {
			strncat(dest, " checked", size - strlen(dest));
			i_set = 1;
		}
		strncat(dest,
			">Read, write or create a file or directory  </td></tr><tr><td><input type=radio name=str_eventid_match value=",
			size - strlen(dest));
		strncat(dest, FILE_ATTRIB_TOKEN, size - strlen(dest));
		if (strcmp
		    (reg_objective.str_eventid_match,
		     FILE_ATTRIB_EVENTS) == 0) {
			strncat(dest, " checked", size - strlen(dest));
			i_set = 1;
		}
		strncat(dest,
			">Modify system, file or directory attributes  </td><td><input type=radio name=str_eventid_match value=",
			size - strlen(dest));
		strncat(dest, PROCESS_TOKEN, size - strlen(dest));
		if (strcmp(reg_objective.str_eventid_match, PROCESS_EVENTS)
		    == 0) {
			strncat(dest, " checked", size - strlen(dest));
			i_set = 1;
		}
		strncat(dest,
			">Start or stop program execution  </td><td><input type=radio name=str_eventid_match value=",
			size - strlen(dest));
		strncat(dest, USER_TOKEN, size - strlen(dest));
		if (strcmp(reg_objective.str_eventid_match, USER_EVENTS) ==
		    0) {
			strncat(dest, " checked", size - strlen(dest));
			i_set = 1;
		}
		strncat(dest,
			">Change user or group identity   </td></tr><tr><td><input type=radio name=str_eventid_match value=",
			size - strlen(dest));
		strncat(dest, SOCKET_TOKEN, size - strlen(dest));
		if (strcmp(reg_objective.str_eventid_match, SOCKET_EVENTS)
		    == 0) {
			strncat(dest, " checked", size - strlen(dest));
			i_set = 1;
		}
		strncat(dest,
			">Establish an outgoing network connection  </td><td><input type=radio name=str_eventid_match value=Any_Event",
			size - strlen(dest));
		if (i_set == 0) {
			strncat(dest, " checked", size - strlen(dest));
			i_set = 1;
		}
		strncat(dest,
			">Any event(s) </td><td></td></tr></table></td></tr>",
			size - strlen(dest));

		strncat(dest,
			"<tr bgcolor=#FFFFCC><td>Event ID Search Term </td><td><input type=text name=str_eventid_text size=50 value=\"",
			size - strlen(dest));
		strncat(dest, reg_objective.str_eventid_match,
			size - strlen(dest));
		strncat(dest, "\"></td></tr>", size - strlen(dest));

		strncat(dest,
			"<tr bgcolor=#FFFFBB><td>Search Term (regular expression) </td><td><input type=text name=str_general_match size=50 value=\"",
			size - strlen(dest));
		strncat(dest, reg_objective.str_general_match,
			size - strlen(dest));
		strncat(dest, "\"></td></tr>", size - strlen(dest));

		strncat(dest,
			"<tr bgcolor=#FFFFCC><td>Select the User Match Type</td><td>",
			size - strlen(dest));
		strncat(dest,
			"<input type=radio name=str_user_match_type value=",
			size - strlen(dest));
		strncat(dest, "Include", size - strlen(dest));
		if (strstr(reg_objective.str_user_match_type, "Exclude") ==
		    NULL)
			strncat(dest, " checked", size - strlen(dest));
		strncat(dest,
			">Include    <input type=radio name=str_user_match_type value=",
			size - strlen(dest));
		strncat(dest, "Exclude", size - strlen(dest));
		if (strstr(reg_objective.str_user_match_type, "Exclude") !=
		    NULL)
			strncat(dest, " checked", size - strlen(dest));
		strncat(dest, ">Exclude    </td></tr>", size - strlen(dest));

		strncat(dest,
			"<tr bgcolor=#FFFFCC><td>User Search Term </td><td><input type=text name=str_user_match size=50 value=\"",
			size - strlen(dest));
		strncat(dest, reg_objective.str_user_match,
			size - strlen(dest));
		strncat(dest, "\"></td></tr>", size - strlen(dest));

		// Identify the event type to capture. Note that there is a table within a table in these radio buttons.
		i_set = 0;

		strncat(dest,
			"<tr bgcolor=#FFFFBB><td>Identify the event types to be captured</td><td><table  width=100% border=0><tr>",
			size - strlen(dest));
		strncat(dest,
			"<td><input type=checkbox name=str_event_type_succ value=",
			size - strlen(dest));
		strncat(dest, SUCCESS_TOKEN, size - strlen(dest));

		if (!strcmp(reg_objective.str_event_type, SUCCESS_TOKEN)
		    || !strcmp(reg_objective.str_event_type, ALL_TOKEN)) {
			strncat(dest, " checked", size - strlen(dest));
			i_set = 1;
		}
		strncat(dest,
			">Success Audit  </td><td><input type=checkbox name=str_event_type_fail value=",
			size - strlen(dest));
		strncat(dest, FAILURE_TOKEN, size - strlen(dest));
		if (!strcmp(reg_objective.str_event_type, FAILURE_TOKEN)
		    || !strcmp(reg_objective.str_event_type, ALL_TOKEN)) {
			strncat(dest, " checked", size - strlen(dest));
			i_set = 1;
		}
		strncat(dest, ">Failure Audit  </td></tr>",
			size - strlen(dest));
		strncat(dest, "</table></td></tr>", size - strlen(dest));

		// Identify the log type to capture. Note that there is a table within a table in these radio buttons.
		i_set = 0;

		strncat(dest,
			"<tr bgcolor=#FFFFCC><td>Identify the event logs <br>(ignored if any objective other <br>than 'Any event(s)' is selected):</td><td><table  width=100% border=0><tr>",
			size - strlen(dest));

		strncat(dest,
			"<tr bgcolor=#FFFFBB><td>Select the Alert Level</td><td>",
			size - strlen(dest));

		// Determine the criticality level
		strncat(dest, "<input type=radio name=str_critic value=",
			size - strlen(dest));
		strncat(dest, CRITICAL_TOKEN, size - strlen(dest));
		if (strstr(reg_objective.str_critic, CRITICAL_TOKEN) != NULL)
			strncat(dest, " checked", size - strlen(dest));
		strncat(dest,
			">Critical    <input type=radio name=str_critic value=",
			size - strlen(dest));
		strncat(dest, PRIORITY_TOKEN, size - strlen(dest));
		if (strstr(reg_objective.str_critic, PRIORITY_TOKEN) != NULL)
			strncat(dest, " checked", size - strlen(dest));
		strncat(dest,
			">Priority    <input type=radio name=str_critic value=",
			size - strlen(dest));
		strncat(dest, WARNING_TOKEN, size - strlen(dest));
		if (strstr(reg_objective.str_critic, WARNING_TOKEN) != NULL)
			strncat(dest, " checked", size - strlen(dest));
		strncat(dest,
			">Warning    <input type=radio name=str_critic value=",
			size - strlen(dest));
		strncat(dest, INFORMATION_TOKEN, size - strlen(dest));
		if (strstr(reg_objective.str_critic, INFORMATION_TOKEN) != NULL)
			strncat(dest, " checked", size - strlen(dest));
		strncat(dest,
			">Information    <input type=radio name=str_critic value=",
			size - strlen(dest));
		strncat(dest, CLEAR_TOKEN, size - strlen(dest));
		if (strstr(reg_objective.str_critic, CLEAR_TOKEN) != NULL)
			strncat(dest, " checked", size - strlen(dest));
		strncat(dest, ">Clear    </td></tr>", size - strlen(dest));

		strncat(dest, "</table><br>", size - strlen(dest));
		strncat(dest, "<input type=hidden name=objnumber value=",
			size - strlen(dest));
		strncat(dest, str_temp_objective, size - strlen(dest));	// Objective number goes here
		strncat(dest,
			"><input type=submit value=\"Change Configuration\">    ",
			size - strlen(dest));
		strncat(dest, "<input type=reset value=\"Reset Form\"></form>",
			size - strlen(dest));
	} else {
		void *rampointer = (void *) NULL;
		char *position;
		char inputbuffer[MAX_AUDIT_CONFIG_LINE];
		int headertype = 0;
		FILE *configfile;

		rampointer = Load_Config_File();
		if (!rampointer) {
			dw_objective_error = WEB_READ_CONFIG_ERROR_CODE;
			dw_objective_delete_error = 1;
		} else {
			int objectivecounter = 0;
			int size = 0;

			position = (char *) rampointer;

			configfile = fopen(CONFIG_FILENAME, "w");
			if (!configfile) {
				strncat(dest,
					"<br><b>NOTE: Could not open the configuration file for writing. Please verify the permissions set on the audit config file.",
					size - strlen(dest));
				Clear_Config_File(rampointer);
				return (0);
			}

			while ((size =
				Grab_RAMConfig_Line(position, inputbuffer,
						    MAX_AUDIT_CONFIG_LINE))) {
				trim(inputbuffer);

				if (headertype == CONFIG_OBJECTIVES) {
					if (objectivecounter ==
					    i_objective_count) {
						// Do not add this line back into the original file.
						position += size;
						objectivecounter++;
						continue;
					}
					objectivecounter++;
				}

				if (!iscomment(inputbuffer)) {
					// Is this line a header?
					if (isheader(inputbuffer)) {
						headertype =
						    getheader(inputbuffer);
					}
				}
				// Print this line to file.
				if (isheader(inputbuffer)
				    || iscomment(inputbuffer)
				    || !strlen(inputbuffer)) {
					fprintf(configfile, "%s\n",
						inputbuffer);
				} else {
					fprintf(configfile, "	%s\n",
						inputbuffer);
				}

				// position+=strlen(inputbuffer)+1;
				position += size;
			}
			if (fclose(configfile)) {
				dw_objective_delete_error = 1;
				strncat(dest,
					"<br><b>NOTE: Could not write to the configuration file. Does the system have enough free disk space?.",
					size - strlen(dest));
				Clear_Config_File(rampointer);
				return (0);
			}

			Clear_Config_File(rampointer);
		}

		if (dw_objective_delete_error == 0)
			strncat(dest,
				"<br><b>The objective has been removed<br><a href=\"/objective\">Return to Objective display</a>.",
				size - strlen(dest));
		else
			strncat(dest,
				"<br>The objective was unable to be deleted.",
				size - strlen(dest));
	}

	return (0);
}

int Grab_RAMConfig_Line(char *source, char *dest, int size)
{
	int count = 0;

	if (!source || !dest || !size) {
		return (0);
	}

	while (*source && count < size) {
		*dest = *source;
		if (*source == '\n') {
			dest++;
			count++;
			break;
		}
		count++;
		dest++;
		source++;
	}
	*dest = '\0';
	if (*source == '\0') {
		return (0);
	}
	return (count);
}

void *Load_Config_File()
{
	FILE *configfile;
	char *location;
	char inputbuffer[MAX_AUDIT_CONFIG_LINE];
	long filesize = 0;
	int configsize = 0;

	configfile = fopen(CONFIG_FILENAME, "r");
	if (!configfile) {
		return ((void *) NULL);
	}

	if (fseek(configfile, 0, SEEK_END)) {
		fclose(configfile);
		return ((void *) NULL);
	}

	filesize = ftell(configfile);
	if (!filesize) {
		fclose(configfile);
		return ((void *) NULL);
	}

	if (fseek(configfile, 0, SEEK_SET)) {
		fclose(configfile);
		return ((void *) NULL);
	}


	location = malloc(filesize + 1);

	if (!location) {
		fclose(configfile);
		return ((void *) NULL);
	}

	while (fgets(inputbuffer, MAX_AUDIT_CONFIG_LINE, configfile)) {
		if ((configsize + strlen(inputbuffer)) > filesize) {
			// We have run out of room.
			free(location);
			fclose(configfile);
			return ((void *) NULL);
		}
		strcpy(location + configsize, inputbuffer);	// Safe from overflow due to above check.
		configsize += strlen(inputbuffer);
	}

	fclose(configfile);

	return (location);
}

void Clear_Config_File(void *location)
{
	if (!location)
		return;

	free(location);
}

int Objective_Result(char *source, char *dest, int size)
{
	// All strncpy or strncat functions in this routine have been designed avoid overflows
	struct Reg_Objective reg_objective;
	int dw_objective_error = 0;
	int i_objective = 0, i_event_type_set = 0;
	char str_obj_count[10], str_eventid_radio[50],
	    str_eventid_text[SIZE_OF_EVENTIDMATCH];
	char *psource = source, Variable[100], Argument[100];

	strncpy(dest,
		"<form action=setobjective><H1><CENTER>SNARE Filtering Objectives Configuration</H1>",
		size);

	strncpy(reg_objective.str_event_type, "",
		sizeof (reg_objective.str_event_type));

	while ((psource =
		GetNextArgument(psource, Variable, sizeof (Variable), Argument,
				sizeof (Argument))) != (char *) NULL) {
		if (strstr(Variable, "str_user_match") != NULL) {
			strncpy(reg_objective.str_user_match, Argument,
				sizeof (reg_objective.str_user_match));
		}

		if (strstr(Variable, "str_general_match") != NULL) {
			strncpy(reg_objective.str_general_match, Argument,
				sizeof (reg_objective.str_general_match));
		}

		if (strstr(Variable, "str_event_type_succ") != NULL) {
			if (i_event_type_set == 1) {
				// A token has already been added
				strncpy(reg_objective.str_event_type, ALL_TOKEN,
					sizeof (reg_objective.str_event_type));
			} else {
				strncat(reg_objective.str_event_type,
					SUCCESS_TOKEN,
					sizeof (reg_objective.str_event_type) -
					strlen(reg_objective.str_event_type));
			}
			i_event_type_set = 1;
		}

		if (strstr(Variable, "str_event_type_fail") != NULL) {
			if (i_event_type_set == 1) {
				// A token has already been added
				// Turn on both success and failure
				strncpy(reg_objective.str_event_type, ALL_TOKEN,
					sizeof (reg_objective.str_event_type));
			} else {
				strncat(reg_objective.str_event_type,
					FAILURE_TOKEN,
					sizeof (reg_objective.str_event_type) -
					strlen(reg_objective.str_event_type));
			}
			i_event_type_set = 1;
		}

		if (strstr(Variable, "str_eventid_match") != NULL) {
			strncpy(str_eventid_radio, Argument,
				sizeof (str_eventid_radio));
		}
		if (strstr(Variable, "str_eventid_text") != NULL) {
			strncpy(str_eventid_text, Argument,
				sizeof (str_eventid_text));
		}
		if (strstr(Variable, "objnumber") != NULL) {
			strncpy(str_obj_count, Argument,
				sizeof (str_obj_count));
		}
		if (strstr(Variable, "str_critic") != NULL) {
			strncpy(reg_objective.str_critic, Argument,
				sizeof (reg_objective.str_critic));
		}
		if (strstr(Variable, "str_user_match_type") != NULL) {
			strncpy(reg_objective.str_user_match_type, Argument,
				sizeof (reg_objective.str_user_match_type));
		}
	}

	if (!i_event_type_set) {
		strncat(dest,
			"<br>Either Success or Failure (or both) mosut be specified. Please hit your 'Back' button to return to the Objective Configuration page.<p>",
			size - strlen(dest));
		return (1);
	}

	if (strstr(str_eventid_radio, "Any_Event") != NULL) {
		strncpy(reg_objective.str_eventid_match, str_eventid_text,
			sizeof (reg_objective.str_eventid_match));
	} else {
		if (strstr(str_eventid_radio, LOGONOFF_TOKEN) != NULL) {
			strncpy(reg_objective.str_eventid_match,
				LOGON_LOGOFF_EVENTS,
				sizeof(reg_objective.str_eventid_match));
		} else if (strstr(str_eventid_radio, FILE_READ_TOKEN) !=
			   NULL) {
			strncpy(reg_objective.str_eventid_match,
				FILE_READ_EVENTS,
				sizeof(reg_objective.str_eventid_match));
		} else if (strstr(str_eventid_radio, FILE_WRITE_TOKEN) !=
			   NULL) {
			strncpy(reg_objective.str_eventid_match,
				FILE_WRITE_EVENTS,
				sizeof(reg_objective.str_eventid_match));
		} else if (strstr(str_eventid_radio, FILE_ATTRIB_TOKEN) !=
			   NULL) {
			strncpy(reg_objective.str_eventid_match,
				FILE_ATTRIB_EVENTS,
				sizeof(reg_objective.str_eventid_match));
		} else if (strstr(str_eventid_radio, PROCESS_TOKEN) !=
			   NULL) {
			strncpy(reg_objective.str_eventid_match,
				PROCESS_EVENTS,
				sizeof(reg_objective.str_eventid_match));
		} else if (strstr(str_eventid_radio, USER_TOKEN) != NULL) {
			strncpy(reg_objective.str_eventid_match,
				USER_EVENTS,
				sizeof(reg_objective.str_eventid_match));
		} else if (strstr(str_eventid_radio, SOCKET_TOKEN) != NULL) {
			strncpy(reg_objective.str_eventid_match,
				SOCKET_EVENTS,
				sizeof(reg_objective.str_eventid_match));
		} else {
			strncpy(reg_objective.str_eventid_match,
				str_eventid_radio,
				sizeof(reg_objective.str_eventid_match));
		}
	}

	if (!dw_objective_error) {

		i_objective = atoi(str_obj_count);

		//-2 = "Add a new objective"
		if (i_objective == -2) {
			void *rampointer = (void *) NULL;
			char *position;
			char inputbuffer[MAX_AUDIT_CONFIG_LINE];
			int headertype = 0;
			FILE *configfile;

			rampointer = Load_Config_File();
			if (!rampointer) {
				dw_objective_error = WEB_READ_CONFIG_ERROR_CODE;
			} else {
				int size = 0;
				int wroteconfig = 0;

				position = (char *) rampointer;

				configfile = fopen(CONFIG_FILENAME, "w");
				if (!configfile) {
					dw_objective_error = 1;
					strncat(dest,
						"<br><b>NOTE: Could not open the configuration file for writing. Please verify the permissions set on the audit config file.",
						size - strlen(dest));
					Clear_Config_File(rampointer);
					return (0);
				}

				while ((size =
					Grab_RAMConfig_Line(position,
							    inputbuffer,
							    MAX_AUDIT_CONFIG_LINE)))
				{
					trim(inputbuffer);

					// Is this line a header?
					if (isheader(inputbuffer)) {
						fprintf(configfile, "%s\n",
							inputbuffer);
						headertype =
						    getheader(inputbuffer);
						if (headertype ==
						    CONFIG_OBJECTIVES) {
							int critic = 0;
							// WRITE OUT NEW OBJECTIVE HERE
							if (strstr
							    (reg_objective.
							     str_critic,
							     CRITICAL_TOKEN)) {
								critic =
								    CRITICALITY_CRITICAL;
							} else
							    if (strstr
								(reg_objective.
								 str_critic,
								 PRIORITY_TOKEN))
							{
								critic =
								    CRITICALITY_PRIORITY;
							} else
							    if (strstr
								(reg_objective.
								 str_critic,
								 WARNING_TOKEN))
							{
								critic =
								    CRITICALITY_WARNING;
							} else
							    if (strstr
								(reg_objective.
								 str_critic,
								 INFORMATION_TOKEN))
							{
								critic =
								    CRITICALITY_INFO;
							} else {
								critic =
								    CRITICALITY_CLEAR;
							}

							fprintf(configfile,
								"	criticality=%d	event=%s	return=%s	user=%s	match=%s\n",
								critic,
								reg_objective.
								str_eventid_match,
								reg_objective.
								str_event_type,
								reg_objective.
								str_user_match,
								reg_objective.
								str_general_match);

							wroteconfig = 1;
						}
					} else {

						// Print this line to file.
						if (iscomment(inputbuffer)
						    || !strlen(inputbuffer)) {
							fprintf(configfile,
								"%s\n",
								inputbuffer);
						} else {
							fprintf(configfile,
								"	%s\n",
								inputbuffer);
						}
					}
					position += size;
				}

				if (!wroteconfig) {
					// Must not have been an objective header in the file...
					int critic = 0;
					char userstring[8];
					// WRITE OUT NEW OBJECTIVE HERE
					if (strstr
					    (reg_objective.str_critic,
					     CRITICAL_TOKEN)) {
						critic = CRITICALITY_CRITICAL;
					} else
					    if (strstr
						(reg_objective.str_critic,
						 PRIORITY_TOKEN)) {
						critic = CRITICALITY_PRIORITY;
					} else
					    if (strstr
						(reg_objective.str_critic,
						 WARNING_TOKEN)) {
						critic = CRITICALITY_WARNING;
					} else
					    if (strstr
						(reg_objective.str_critic,
						 INFORMATION_TOKEN)) {
						critic = CRITICALITY_INFO;
					} else {
						critic = CRITICALITY_CLEAR;
					}

					if (strstr
					    (reg_objective.str_user_match_type,
					     "Exclude") == NULL) {
						strncpy(userstring, "user",
							sizeof (userstring));
					} else {
						strncpy(userstring, "user!",
							sizeof (userstring));
					}

					fprintf(configfile,
						"\n[Objectives]\n	criticality=%d	event=%s	return=%s	%s=%s	match=%s\n",
						critic,
						reg_objective.
						str_eventid_match,
						reg_objective.
						str_event_type,
						userstring,
						reg_objective.
						str_user_match,
						reg_objective.
						str_general_match);
				}

				if (fclose(configfile)) {
					dw_objective_error = 1;
					strncat(dest,
						"<br><b>NOTE: Could not write to the configuration file. Does the system have enough free disk space?.",
						size - strlen(dest));
					Clear_Config_File(rampointer);
					return (0);
				}

				Clear_Config_File(rampointer);
			}
		} else {
			// Modify an existing objective
			void *rampointer = (void *) NULL;
			char *position;
			char inputbuffer[MAX_AUDIT_CONFIG_LINE];
			int headertype = 0;
			FILE *configfile;

			rampointer = Load_Config_File();
			if (!rampointer) {
				dw_objective_error = WEB_READ_CONFIG_ERROR_CODE;
			} else {
				int objectivecounter = 0;
				int size = 0;

				position = (char *) rampointer;

				configfile = fopen(CONFIG_FILENAME, "w");
				if (!configfile) {
					dw_objective_error = 1;
					strncat(dest,
						"<br><b>NOTE: Could not open the configuration file for writing. Please verify the permissions set on the audit config file.",
						size - strlen(dest));
					Clear_Config_File(rampointer);
					return (0);
				}

				while ((size =
					Grab_RAMConfig_Line(position,
							    inputbuffer,
							    MAX_AUDIT_CONFIG_LINE)))
				{
					trim(inputbuffer);

					if (headertype == CONFIG_OBJECTIVES) {
						if (objectivecounter ==
						    i_objective) {
							// Replace this objective with the new version.
							int critic = 0;
							// WRITE OUT NEW OBJECTIVE HERE
							if (strstr
							    (reg_objective.
							     str_critic,
							     CRITICAL_TOKEN)) {
								critic =
								    CRITICALITY_CRITICAL;
							} else
							    if (strstr
								(reg_objective.
								 str_critic,
								 PRIORITY_TOKEN))
							{
								critic =
								    CRITICALITY_PRIORITY;
							} else
							    if (strstr
								(reg_objective.
								 str_critic,
								 WARNING_TOKEN))
							{
								critic =
								    CRITICALITY_WARNING;
							} else
							    if (strstr
								(reg_objective.
								 str_critic,
								 INFORMATION_TOKEN))
							{
								critic =
								    CRITICALITY_INFO;
							} else {
								critic =
								    CRITICALITY_CLEAR;
							}

							fprintf(configfile,
								"	criticality=%d	event=%s	return=%s	user=%s	match=%s\n",
								critic,
								reg_objective.
								str_eventid_match,
								reg_objective.
								str_event_type,
								reg_objective.
								str_user_match,
								reg_objective.
								str_general_match);

							position += size;
							objectivecounter++;
							continue;
						}
						objectivecounter++;
					}

					if (!iscomment(inputbuffer)) {
						// Is this line a header?
						if (isheader(inputbuffer)) {
							headertype =
							    getheader
							    (inputbuffer);
						}
					}
					// Print this line to file.
					if (isheader(inputbuffer)
					    || iscomment(inputbuffer)
					    || !strlen(inputbuffer)) {
						fprintf(configfile, "%s\n",
							inputbuffer);
					} else {
						fprintf(configfile, "	%s\n",
							inputbuffer);
					}

					// position+=strlen(inputbuffer)+1;
					position += size;
				}

				if (fclose(configfile)) {
					dw_objective_error = 1;
					strncat(dest,
						"<br><b>NOTE: Could not write to the configuration file. Does the system have enough free disk space?.",
						size - strlen(dest));
					Clear_Config_File(rampointer);
					return (0);
				}

				Clear_Config_File(rampointer);
			}
		}

		if (dw_objective_error == 0)
			strncat(dest,
				"<br>The objective has been modifed/added.",
				size - strlen(dest));
		else
			strncat(dest,
				"<br>The objective was unable to be modifed/added.",
				size - strlen(dest));
	}

	return (0);
}

int DefaultHeader(char *source, char *dest, int size)
{
	strncpy(dest, "<HTML><head>"
		"<title>InterSect Alliance - Information Technology Security</title>"
		"<meta name=\"TITLE\" content=\"InterSect Alliance - Information Technology Security\">"
		"</head>"
		"<body text=black bgcolor=white link=#000066 vlink=#000044 alink=#000055>"
		"<table border=0 cellspacing=0 cellpadding=0 columns=3 width=100%>"
		"<tbody>"
		"<tr>"
		"<td height=70><img src=/intersect.gif alt=\"InterSect\" width=205 height=70 hspace=20 vspace=0 border=0 align=Right>"
		"</td>"
		"<td height=70 width=1% bgcolor=#eeeeee><br></td>"
		"<td height=70 width=1% bgcolor=#dddddd><br></td>"
		"<td height=70 width=1% bgcolor=#cccccc><br></td>"
		"<td height=70 width=1% bgcolor=#ccbbbb><br></td>"
		"<td height=70 width=1% bgcolor=#ccaaaa><br></td>"
		"<td height=70 width=1% bgcolor=#cc9999><br></td>"
		"<td height=70 width=1% bgcolor=#cc8888><br></td>"
		"<td height=70 width=1% bgcolor=#cc7777><br></td>"
		"<td height=70 width=1% bgcolor=#cc6666><br></td>"
		"<td height=70 width=1% bgcolor=#cc5555><br></td>"
		"<td height=70 width=1% bgcolor=#cc4444><br></td>"
		"<td height=70 width=1% bgcolor=#cc3333><br></td>"
		"<td height=70 width=1% bgcolor=#cc2222><br></td>"
		"<td height=70 width=1% bgcolor=#cc1111><br></td>"
		"<td height=70 width=86% bgcolor=#cc0000><div align=right><font face=\"Helvetica,Arial,sans-serif\" size=+2 color=white><b><center>SNARE for Solaris&nbsp;</b></font><font color=white></font>&nbsp;&nbsp;</center></div></td>"
		"</tr>"
		"</tbody>"
		"</table>"
		"<table border=0 cellspacing=0 cellpadding=0 columns=1 width=100%>"
		"<tr><td height=3 border=0 bgcolor=white width=100%></td></tr>"
		"<tr><td height=3 border=0 bgcolor=black width=100%></td></tr>"
		"<tr><td height=2 border=0 bgcolor=#AAAAAA width=100%></td></tr>"
		"<tr><td height=2 border=0 bgcolor=white width=100%></td></tr>"
		"</table>"
		"<table border=0 cellspacing=0 cellpadding=5 columns=2 width=100% height=100%>"
		"<tbody>"
		"<tr>"
		"<td valign=Top width=20% bgcolor=#cc0000>"
		"<div align=Center><font color=#ffffff>"
		"<br>"
		"<font face=\"Helvetica,Arial,sans-serif\" size=-1><B>"
		"<br><A HREF=\"/network\" style=\"color:FFFFFF;text-decoration:none\">Network Configuration</A><br>"
		"<br><A HREF=\"/remote\" style=\"color:FFFFFF;text-decoration:none\">Remote Control Configuration</A><br>"
		"<br><A HREF=\"/objective\" style=\"color:FFFFFF;text-decoration:none\">Objectives Configuration</A><br>"
		"<br><A HREF=\"/status\" style=\"color:FFFFFF;text-decoration:none\">View Audit Service Status</A><br>"
		"<br><A HREF=\"/restart\" style=\"color:FFFFFF;text-decoration:none\">Apply the Latest Audit Configuration</A><br>"
		"<br><br><font size=-2><A HREF=\"/Users\" style=\"color:FFFFFF;text-decoration:none\">Display a list of Users</A><br>"
		"<br><A HREF=\"/Groups\" style=\"color:FFFFFF;text-decoration:none\">Display a list of Groups</A><br>"
		"<br><A HREF=\"/GroupMembers\" style=\"color:FFFFFF;text-decoration:none\">Display a list of GroupMembers</A></font><br>"
		"</b></font>"
		"<br>"
		"</div>"
		"</td>"
		"<td width=100% valign=Top>"
		"<table cellpadding=0 cellspacing=10 border=0 width=100%>"
		"<tbody>" "<tr>" "<td valign=Top align=Justify>", size);

	return (0);
}

int DefaultFooter(char *source, char *dest, int size)
{
	strncpy(dest, "</td>"
		"</tr>"
		"</tbody>"
		"</table>"
		"</body><center>"
		"<BR><BR><FONT SIZE=-1 face=helvetica>(c) <A HREF=\"http://www.intersectalliance.com\">Intersect Alliance</A> Pty Ltd 2002. "
		"This site is powered by <A HREF=\"http://www.intersectalliance.com/projects/index.html\">SNARE for Solaris.</A></FONT>"
		"</center></html>", size);

	return (0);
}

int DisplayTextHeader(int http_socket)
{
	char HTTPBuffer[512];
	// Overwrite HTTPBuffer with the header data.
	strncpy(HTTPBuffer, "HTTP/1.0 200 OK\r\n"
		"Server: SNARE/1.0\r\n"
		"MIME-version: 1.0\r\n"
		"Content-type: text/plain\r\n\r\n", sizeof (HTTPBuffer));
	return (Send(http_socket, HTTPBuffer, strlen(HTTPBuffer)));
}

int Restart(char *source, char *dest, int size, int socketone, int sockettwo)
{
	int pid;

	switch (pid = fork()) {
	case 0:
		// Child process.
		// close our copy of the existing sockets.
		close(sockettwo);
		close(socketone);
		sleep(1);
		execlp("/etc/init.d/snare", "snare", "restart", "&",
		       (char *) 0);
	default:
		strncpy(dest,
			"<HTML><BODY><H1><CENTER>Reapply the Latest Configuration</H1><P>Reapplying the configuration</CENTER></BODY></HTML>",
			size);
	}

	return (0);
}

// General instructions:
// $ expr `./base64 intersect.gif | wc -c` - `./base64 intersect.gif | wc -l` "*" 2 + 1
//   2029
// ./base64 intersect.gif | sed 's/^/   "/' | sed 's/.$/"  \\/' >out.txt
// 
//  mangle out.txt as appropriate - add the (wc -c) - ((wc -l)*2) + 1 figure to the buffer size.
int InterSectImage(char *source, char *dest, int size)
{
	char IntersectGif[2029];
	char temp[2029];
	int size2;

	strncpy(IntersectGif,
		"R0lGODlhzQBGALMAAB8aF01JR2toZtwrGYF+fONVRo+Mi5eWleuCePKzrLOysszLy/jRzunl"
		"5f////j29SwAAAAAzQBGAEAI/gAdCBxIsKDBgw4eIFxIUCHDhxAjSkT4AMGAAQgmatw40CHH"
		"jxs9ghQJsuTEBxdTpkxgsqXLlzBjyvyoMuXMmzAB6NQp0MDOnzoDkOz504ADn0CTAniAVOlO"
		"oQqcSgVqdKpVAARrXiyA0+XQkkCJ7jRakMDPAA3MjmUYtikAsgfV6lTw9WFYgwKACj34gIFW"
		"jF0DCx5MeGPUsw08WtTKMqFcAGgZNghQ9ChQAQ1Kuq36s7DnzxL7/kXw4DFah6gFpk4o8ADV"
		"Bwpjs45d1/Lau6Bz6y5YQCvXmbV3Cx++MUHvvxcZHFSgoOGB5g4WGFhAfUFmAwqqL2i94EGD"
		"AwSZ/nNXreDA9gXmq5vvmb165sDBIcYnfjAB8gLz6ev3up+ifa0i5VeYgCEtROBIsyUIm0Gy"
		"seZgbatFqNqEBB2nUkb9BXbXZgJN9hN0YukE10FtVYbQY1clxVmKSjW0WE0HZijRhiYetIBS"
		"AdjVGYcUPZAXi0Bth9tBHv4EnkEWqjQUU5SxKMB2HTEFpJBXrTiVjFhO9F9NGKIIpE7g8ahZ"
		"ZUNmaeZGL6rUkQJNfgllQmyyCKJBHJZ55p0LNZBkSr8VhN4D66mGnkGBEjRdRwcUetQB7zmw"
		"HnraBYqddo3ieecDe2IUo6WcdtrRlkp6KuqoqmH6F34ZbkrqpZkOoOqq/7DqZmpNjcVqK55/"
		"VSrjqyR2BmdSkSHEoVtS2TbVXkV+CQB0yho5EKgp6XorQmFJudZjBJAkZq87GfvWQo/NKWyN"
		"8jnQwJDQXlTrtNT6KuYDN/6UmLcjGlTiWic2K68D+s5VEEq+Ycmrve6S21GbKrK1o8Edndsv"
		"AN3ZqRrCAAiAUJopKcfuTQ0oYMChGwsU709vCuRXwAMl2+x7FF9FpVVWFhsyfMRaPBByDj0m"
		"wMc898wzwgZYK2JLdfo6s4ytKqSzz0z3zGiIbw1cdLdH75rpbz/qC2XWci40NU9Vy4jxRWGX"
		"/dnYA6zLWgPW8fUvTEtS5KDZoD2QLp/LpYceiP6DFqRoT1B+pxBzDk33HV2PHkloon/TLRhy"
		"KzkuuWB9JYnh5C8N7OndgOmnOeaUN3DfgqCb9DmenDd6eulho315qpfSLRpjq7NOd6av244g"
		"f2f+RXpH+9UON+h/6W78RpmqPXfIuMHbZAAEzMvXsCkKLdVesBnQslRPDtSv4g6MjWqWtd/l"
		"MLDSQm0UsU5565RQD2zPYmYPL9XhXwl4JHx/NJ71k83A2xbBuiVAgpgmfSkDmmGGhLb9Yal/"
		"Q3MAwsBHL4URkGEDWVrTeqbADW6QYvo7FfliAsFvdShIhcOg9xaGr7jUDzwPyxFB7pY/s5Ww"
		"XocJynVUKJB7RZBBGv70oNPeQxWfcU1E3TlIAx24nxsy6IhUsSBPCphBxDyIQqvpodEKckQZ"
		"HkSEsMtJwVpYkPMlpWQDnCIPHQDFKfFri2WkWL1MNpoeXfGOEIINbRS0PBKO8YfL0QsCV3hB"
		"MiIkTsqKGBwNYkZ/FQRtDBCJyoAEovi56QEpiln7jheRHO4EjUkbiCebZZSRTQmTVXqjVTg5"
		"EdecxSMnq0mfwgUR6j1FamRaJCs5ogAC9OxNaGuMl760vqLg8ja63CXcTqW0+u3QkBz5GlaU"
		"2RVIluZDH3GLAAg0FGlSsysA45K56tedRqaILuNCJtW+eRPxqaaNKUqiJSlZF2+ys53FA6ne"
		"PY3nzgLtM2TeqeM/jxdLGDHIY+Jim0GsQ5K2pQyNDhVZZuClnTdVlKED1UhsGNCqAWjMOdsp"
		"j0ekQyc0OupNzBHcQJ5zKECdZ0QONQ8TOVkRyA1AV94xAOMSZYD3DEokMvVbEj2203mt5wHM"
		"OVTfBJId7ugvoxJBm0qUZ66IdqhtbKNUdCoq0ehUCl6ZsWrbKKodh1x0kFAliFQL8NG0lm5L"
		"CJAe3QICADs=", 2029);

	size2=base64decode (temp, IntersectGif);

	memcpy(dest,temp,size2);
	return(size2);
}

int debracket(char *source, char *dest, int length)
{
	// This routine is simply to replace the HTML metacharacters "<" and ">" with
	// "&lt;" and "&gt;". That's all!

	int count = 0;
	char *copyofdest;
	copyofdest = dest;

	if (!source || !dest)
		return (1);

	while (*source) {
		if (*source == '<') {
			if (count < length) {
				*dest = '&';
				dest++;
				count++;
			}
			if (count < length) {
				*dest = 'l';
				dest++;
				count++;
			}
			if (count < length) {
				*dest = 't';
				dest++;
				count++;
			}
			if (count < length) {
				*dest = ';';
				dest++;
				count++;
			}
		} else if (*source == '>') {
			if (count < length) {
				*dest = '&';
				dest++;
				count++;
			}
			if (count < length) {
				*dest = 'g';
				dest++;
				count++;
			}
			if (count < length) {
				*dest = 't';
				dest++;
				count++;
			}
			if (count < length) {
				*dest = ';';
				dest++;
				count++;
			}
		} else {
			if (count < length) {
				*dest = *source;
				dest++;
				count++;
			}
		}
		source++;
	}
	*dest = '\0';

	dest = copyofdest;

	return (0);
}

int Read_Config_From_File(struct Reg_Config *config_struct)
{
	FILE *configfile;
	char inputbuffer[MAX_AUDIT_CONFIG_LINE];
	int headertype = 0;

	configfile = fopen(CONFIG_FILENAME, "r");
	if (!configfile)
		return (0);

	strcpy(config_struct->str_ClientName, "");
	strcpy(config_struct->str_NetworkDestination, "");
	strcpy(config_struct->str_FileDestination, "");
	config_struct->dw_stdout = 0;
	config_struct->dw_DestPort = 0;

	while (fgets(inputbuffer, MAX_AUDIT_CONFIG_LINE, configfile)) {
		// Kill whitespace from start and end of line.
		trim(inputbuffer);

		if (!iscomment(inputbuffer)) {
			// Is this line a header?
			if (isheader(inputbuffer)) {
				headertype = getheader(inputbuffer);
			} else {
				if (headertype == CONFIG_HOSTID) {
					if (!strlen
					    (config_struct->str_ClientName)) {
						// Grab hostname here.
						if (!gethostident
						    (inputbuffer,
						     config_struct->
						     str_ClientName,
						     SIZE_OF_CLIENTNAME)) {
							strncpy(config_struct->
								str_ClientName,
								"",
								SIZE_OF_CLIENTNAME);
						}
					}
				} else if (headertype == CONFIG_OUTPUT) {
					if (iswebfilename(inputbuffer)) {
						if (!getconfstring
						    (inputbuffer,
						     config_struct->
						     str_FileDestination,
						     SIZE_OF_DESTINATION)) {
							strncpy(config_struct->
								str_FileDestination,
								"",
								SIZE_OF_DESTINATION);
						}
					} else if (isnetwork(inputbuffer)) {
						config_struct->dw_DestPort =
						    getnetwork(inputbuffer,
							       config_struct->
							       str_NetworkDestination,
							       SIZE_OF_DESTINATION);
						if (!config_struct->dw_DestPort) {
							strncpy(config_struct->
								str_NetworkDestination,
								"",
								SIZE_OF_DESTINATION);
						}
					} else if (isstdout(inputbuffer)) {
						config_struct->dw_stdout = 1;
					}
				}
			}
		}
	}
	// If we have made it this far, then we don't have any objectives.
	fclose(configfile);
	return (0);

}

int Read_Remote_From_File(struct Reg_Remote *remote_struct)
{
	FILE *configfile;
	char inputbuffer[MAX_AUDIT_CONFIG_LINE];
	int headertype = 0;

	configfile = fopen(CONFIG_FILENAME, "r");
	if (!configfile)
		return (0);

	remote_struct->dw_Allow = 0;
	remote_struct->dw_WebPort = 80;
	remote_struct->dw_Restrict = 0;
	strncpy(remote_struct->str_RestrictIP, "", SIZE_OF_RESTRICTIP);
	remote_struct->dw_Password = 0;
	strncpy(remote_struct->str_Password, "", SIZE_OF_PASSWORD);	// Just a failsafe password - not a default.

	while (fgets(inputbuffer, MAX_AUDIT_CONFIG_LINE, configfile)) {
		// Kill whitespace from start and end of line.
		trim(inputbuffer);

		if (!iscomment(inputbuffer)) {
			// Is this line a header?
			if (isheader(inputbuffer)) {
				headertype = getheader(inputbuffer);
			} else {
				if (headertype == CONFIG_REMOTE) {
					if (regmatchi(inputbuffer, "^allow=")) {
						if (regmatchi
						    (inputbuffer, "=1")) {
							remote_struct->
							    dw_Allow = 1;
						}
					} else
					    if (regmatchi
						(inputbuffer,
						 "^listen_port=")) {
						remote_struct->dw_WebPort =
						    getport(inputbuffer);
					} else
					    if (regmatchi
						(inputbuffer,
						 "^restrict_ip=")) {
						if (!getconfstring
						    (inputbuffer,
						     remote_struct->
						     str_RestrictIP,
						     SIZE_OF_RESTRICTIP)) {
							remote_struct->
							    dw_Restrict = 0;
							strncpy(remote_struct->
								str_RestrictIP,
								"",
								SIZE_OF_RESTRICTIP);
						} else {
							remote_struct->
							    dw_Restrict = 1;
						}
					} else
					    if (regmatchi
						(inputbuffer, "^accesskey=")) {
						if (!getconfstring
						    (inputbuffer,
						     remote_struct->
						     str_Password,
						     SIZE_OF_PASSWORD)) {
							remote_struct->
							    dw_Password = 0;
							strncpy(remote_struct->
								str_Password,
								"",
								SIZE_OF_PASSWORD);
						} else {
							remote_struct->
							    dw_Password = 1;
						}
					}
				}
			}
		}
	}

	fclose(configfile);
	return (0);

}

// Return the host identifier
int getnetwork(char *string, char *host, int length)
{
	char *stringp = string;
	char *pos;
	char strPort[10];

	stringp = strstr(string, "=");

	if (stringp != (char *) NULL) {
		stringp++;
		if (strlen(stringp)) {
			pos = strstr(stringp, ":");
			if (pos != (char *) NULL) {
				strncpy(strPort, pos + 1, 10);
				*pos = '\0';
				strncpy(host, stringp, length - 1);
				return (atoi((char *) strPort));
			}
		} else {
			return (0);
		}
	} else {
		return (0);
	}
	return(0);
}

int iswebfilename(char *string)
{
	if (regmatch(string, "^file=")) {
		if (!regmatchi(string, "^file=stdout")) {
			return (1);
		} else {
			return (0);
		}
	} else {
		return (0);
	}
}

int isstdout(char *string)
{
	if (regmatchi(string, "^file=stdout")) {
		return (1);
	} else {
		return (0);
	}
}

FILE *Find_First_Objective()
{
	FILE *configfile;
	char inputbuffer[MAX_AUDIT_CONFIG_LINE];
	int headertype = 0;

	configfile = fopen(CONFIG_FILENAME, "r");
	if (!configfile)
		return (configfile);

	while (fgets(inputbuffer, MAX_AUDIT_CONFIG_LINE, configfile)) {
		// Kill whitespace from start and end of line.
		trim(inputbuffer);

		if (!iscomment(inputbuffer)) {
			// Is this line a header?
			if (isheader(inputbuffer)) {
				headertype = getheader(inputbuffer);
				if (headertype == CONFIG_OBJECTIVES) {
					return (configfile);
				}
			}
		}
	}
	// If we have made it this far, then we don't have any objectives.
	fclose(configfile);
	configfile = (FILE *) NULL;
	return (configfile);
}

int Get_Next_Objective(FILE * configfile, struct Reg_Objective *objective)
{
	char inputbuffer[MAX_AUDIT_CONFIG_LINE];
	// Save off enough space to store the data we need.
	char event[MAX_AUDIT_CONFIG_LINE], user[MAX_AUDIT_CONFIG_LINE],
	    path[MAX_AUDIT_CONFIG_LINE];
	int criticality;
	int returncode;
	int excludeflag = 0;
	//int returnflag;		// HERE: Still need to add in support for this.
	//int notreturnflag = 0;	// HERE: Still need to add in support for this.

	while (fgets(inputbuffer, MAX_AUDIT_CONFIG_LINE, configfile)) {
		if (isheader(inputbuffer)) {
			return (0);
		}

		if ((criticality =
		     splitobjective(inputbuffer, event, user, path,
				    &returncode, &excludeflag)) > -1) {
			// add the objective to the linked list.
			trimallwhitespace(event);

			strncpy(objective->str_general_match, path,
				SIZE_OF_GENERALMATCH);
			strncpy(objective->str_eventid_match, event,
				SIZE_OF_EVENTIDMATCH);

			if (excludeflag) {
				strncpy(objective->str_user_match_type,
					"Exclude",
					sizeof (objective->
						str_user_match_type));
			} else {
				strncpy(objective->str_user_match_type,
					"Include",
					sizeof (objective->
						str_user_match_type));
			}
			strncpy(objective->str_user_match, user,
				SIZE_OF_USERMATCH);
			if (returncode == 1) {
				strncpy(objective->str_event_type,
					SUCCESS_TOKEN, SIZE_OF_EVENTLOG);
			} else if (returncode == 0) {
				strncpy(objective->str_event_type,
					FAILURE_TOKEN, SIZE_OF_EVENTLOG);
			} else {
				strncpy(objective->str_event_type, ALL_TOKEN,
					SIZE_OF_EVENTLOG);
			}

			if (criticality == CRITICALITY_CRITICAL) {
				strncpy(objective->str_critic, CRITICAL_TOKEN,
					SIZE_OF_CRITICALITY);
			} else if (criticality == CRITICALITY_PRIORITY) {
				strncpy(objective->str_critic, PRIORITY_TOKEN,
					SIZE_OF_CRITICALITY);
			} else if (criticality == CRITICALITY_WARNING) {
				strncpy(objective->str_critic, WARNING_TOKEN,
					SIZE_OF_CRITICALITY);
			} else if (criticality == CRITICALITY_INFO) {
				strncpy(objective->str_critic,
					INFORMATION_TOKEN, SIZE_OF_CRITICALITY);
			} else {
				strncpy(objective->str_critic, CLEAR_TOKEN,
					SIZE_OF_CRITICALITY);
			}
			return (1);
		}
	}
	return (0);
}

int Close_Objective_File(FILE * configfile)
{
	return (fclose(configfile));
}
